#include <libwccl/ops/match/conditions/tokencondition.h>
#include <libwccl/values/tokenmatch.h>

namespace Wccl {


TokenCondition::TokenCondition(const boost::shared_ptr<Function<Bool> >& predicate)
	: _predicate(predicate)
{
}

std::string TokenCondition::name() const
{
	return _predicate->raw_name();
}

std::string TokenCondition::to_string(const Corpus2::Tagset& tagset) const
{
	return _predicate->to_string(tagset);
}

MatchResult TokenCondition::apply(boost::shared_ptr<Position> iter_pos, const FunExecContext& context) const
{
	if (_predicate->apply(context)->get_value()) {
		boost::shared_ptr<Match> match = boost::shared_ptr<Match>(new TokenMatch(
				context.sentence_context().get_abs_position(*iter_pos)));
		// increase the $_ variable by one after successful token match
		iter_pos->set_value(iter_pos->get_value() + 1);
		return MatchResult(true, match);
	}
	else {
		return MatchResult();
	}
}

std::ostream& TokenCondition::write_to(std::ostream& os) const
{
	return os << *_predicate;
}

} /* end ns Wccl */
